"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefinitionsResolver = void 0;
const lodash_es_1 = require("lodash-es");
const SchemaResolver_1 = require("./SchemaResolver");
const specifications_1 = require("../utils/specifications");
const generators_1 = require("../utils/generators");
const formatters_1 = require("../utils/formatters");
const constants_1 = require("../constants");
class DefinitionsResolver {
    constructor(components) {
        this.components = components;
        this.scanDefinitions = () => {
            const results = {};
            const requestBodies = (0, lodash_es_1.get)(this.components, "requestBodies");
            const schemas = (0, lodash_es_1.get)(this.components, "schemas");
            (0, lodash_es_1.forEach)(requestBodies, (requestBody, requestBodyName) => {
                if ((0, specifications_1.isRequestBody)(requestBody)) {
                    return (results[requestBodyName] = SchemaResolver_1.SchemaResolver.of({
                        results,
                        schema: (0, lodash_es_1.get)(requestBody, "content.application/json.schema"),
                        key: requestBodyName,
                        parentKey: requestBodyName,
                    })
                        .resolve()
                        .getSchemaType());
                }
                return (results[requestBodyName] = SchemaResolver_1.SchemaResolver.of({
                    results,
                    schema: requestBody,
                    key: requestBodyName,
                    parentKey: requestBodyName,
                })
                    .resolve()
                    .getSchemaType());
            });
            (0, lodash_es_1.forEach)(schemas, (schema, schemaName) => {
                return (results[schemaName] = SchemaResolver_1.SchemaResolver.of({
                    results,
                    schema: schema,
                    key: schemaName,
                    parentKey: schemaName,
                })
                    .resolve()
                    .getSchemaType());
            });
            this.resolvedDefinitions = results;
            return this;
        };
        this.toDeclarations = () => {
            const arr = Object.keys(this.resolvedDefinitions)
                .sort()
                .map((key) => {
                if ((0, lodash_es_1.includes)(key, constants_1.ENUM_SUFFIX)) {
                    return (0, generators_1.generateEnums)(this.resolvedDefinitions, key);
                }
                if (this.resolvedDefinitions[key] === "object" || (0, lodash_es_1.isEmpty)(this.resolvedDefinitions[key])) {
                    return `export interface ${(0, formatters_1.addPrefixForInterface)((0, formatters_1.toCapitalCase)(key))} {[key:string]:any}`;
                }
                const val = (0, formatters_1.toTypes)(this.resolvedDefinitions[key]);
                if (val) {
                    return `export interface ${(0, formatters_1.addPrefixForInterface)((0, formatters_1.toCapitalCase)(key))} ${val}`;
                }
            });
            return (0, lodash_es_1.compact)(arr);
        };
    }
    static of(components) {
        return new DefinitionsResolver(components);
    }
}
exports.DefinitionsResolver = DefinitionsResolver;
