"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertJsonToString = exports.toTypes = exports.arrayToObject = exports.addPrefixForInterface = exports.toCapitalCase = void 0;
const lodash_es_1 = require("lodash-es");
const specifications_1 = require("./specifications");
const constants_1 = require("../constants");
const toCapitalCase = (str) => {
    if (!str) {
        return "";
    }
    const camelStr = (0, lodash_es_1.camelCase)(str);
    return `${camelStr.charAt(0).toUpperCase()}${camelStr.slice(1)}`;
};
exports.toCapitalCase = toCapitalCase;
const addPrefix = (prefix) => (str = "") => `${prefix}${str}`;
exports.addPrefixForInterface = addPrefix("I");
const arrayToObject = (arr = []) => {
    let obj = {};
    (0, lodash_es_1.forEach)(arr, (item) => {
        obj[item] = item;
    });
    return obj;
};
exports.arrayToObject = arrayToObject;
const toTypes = (definitions) => {
    if ((0, lodash_es_1.isEmpty)(definitions)) {
        return;
    }
    const fieldDefinitionList = (0, lodash_es_1.map)(definitions, (value, key) => {
        if ((0, specifications_1.isObject)(value) && Object.keys(value).length === 1 && !(0, lodash_es_1.isEmpty)((0, lodash_es_1.pickBy)(value, (type) => type === "FormData"))) {
            return `${convertKeyToCamelCaseAndAddQuote(key)}: FormData`;
        }
        return (0, specifications_1.isObject)(value)
            ? `${convertKeyToCamelCaseAndAddQuote(key)}: ${JSON.stringify(value).replace(/"/g, "")};`
            : `${convertKeyToCamelCaseAndAddQuote(key)}: ${(0, lodash_es_1.replace)(value, constants_1.ENUM_SUFFIX, "")};`;
    });
    return (definitions &&
        `{
        ${fieldDefinitionList.sort().join("\n")}
      }`);
};
exports.toTypes = toTypes;
const convertKeyToCamelCaseAndAddQuote = (key) => {
    const isOptional = (0, lodash_es_1.indexOf)(key, "?") > -1;
    return `'${(0, lodash_es_1.camelCase)((0, exports.toCapitalCase)((0, lodash_es_1.trimEnd)(key, "?")))}'${isOptional ? "?" : ""}`;
};
const convertJsonToString = (str, errorMsg = constants_1.ERROR_MESSAGES.INVALID_JSON_FILE_ERROR, output = console.error) => {
    if (typeof str !== "string") {
        return;
    }
    try {
        return JSON.parse(str);
    }
    catch (e) {
        output(errorMsg);
        return;
    }
};
exports.convertJsonToString = convertJsonToString;
