"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateResponseType = exports.generateHeader = exports.generateRequestArguments = exports.generateClientName = exports.generateFunctionName = exports.generateEnums = void 0;
const lodash_es_1 = require("lodash-es");
const specifications_1 = require("./specifications");
const constants_1 = require("../constants");
const formatters_1 = require("./formatters");
const generateEnums = (definitions, key) => {
    if ((0, lodash_es_1.isEmpty)(definitions)) {
        return "";
    }
    const enums = definitions[key];
    const hasNumber = (0, lodash_es_1.some)(enums, (enumValue) => (0, specifications_1.isNumber)(enumValue));
    const enumName = (0, lodash_es_1.replace)(key, constants_1.ENUM_SUFFIX, "");
    return hasNumber
        ? `export type ${enumName} = ${enums.map((item) => JSON.stringify(item)).join("|")}`
        : `export enum ${enumName} ${JSON.stringify((0, formatters_1.arrayToObject)(enums)).replace(/:/gi, "=")}`;
};
exports.generateEnums = generateEnums;
const generateFunctionName = (method, operationId) => method === "get" ? `use${(0, formatters_1.toCapitalCase)((0, lodash_es_1.camelCase)(operationId))}Request` : `${(0, lodash_es_1.camelCase)(operationId)}Request`;
exports.generateFunctionName = generateFunctionName;
const generateClientName = (method, responseType) => method === "get"
    ? `useRequest<${responseType || undefined}, IResponseError>`
    : `client.request<${responseType || undefined}, AxiosResponse<${responseType || undefined}>>`;
exports.generateClientName = generateClientName;
// TODO: 1.refactor THeader logic to align with resolvedPath.xxxParams
// TODO: 2.add response type for download file
const generateRequestArguments = (resolvedPath) => {
    const argumentTypes = !(0, lodash_es_1.isEmpty)(Object.assign(Object.assign({}, resolvedPath.TReq), resolvedPath.THeader))
        ? (0, formatters_1.toTypes)(Object.assign(Object.assign({}, resolvedPath.TReq), resolvedPath.THeader))
        : undefined;
    const requestParamList = (0, lodash_es_1.compact)([
        ...resolvedPath.pathParams,
        ...resolvedPath.queryParams,
        ...resolvedPath.bodyParams,
        ...resolvedPath.formDataParams,
        ...Object.keys(resolvedPath.THeader),
        resolvedPath.requestBody,
    ]).map((param) => (0, lodash_es_1.camelCase)(param));
    const requestParams = requestParamList.length === 0 ? "" : `{${requestParamList.join(",")}}:${argumentTypes}`;
    return resolvedPath.method === "get"
        ? `${requestParams ? requestParams + ", " : ""}SWRConfig?: ISWRConfig<${resolvedPath.TResp || undefined}, IResponseError>, axiosConfig?: AxiosRequestConfig`
        : `${requestParams ? requestParams + ", " : ""}axiosConfig?: AxiosRequestConfig`;
};
exports.generateRequestArguments = generateRequestArguments;
const generateHeader = (hasBody, contentTypes, operationId, header) => {
    const result = (0, lodash_es_1.reduce)(header, (result, _, key) => result + `"${key}": ` + (0, lodash_es_1.camelCase)(key) + ", ", "");
    const contentType = hasBody ? `"Content-Type": "${(0, lodash_es_1.get)(contentTypes, operationId !== null && operationId !== void 0 ? operationId : "", "application/json")}"` : "";
    return `headers: { ${result}${contentType}},`;
};
exports.generateHeader = generateHeader;
const generateResponseType = (axiosHeaderConfig) => axiosHeaderConfig.includes('"Accept":') ? 'responseType: "blob",' : "";
exports.generateResponseType = generateResponseType;
